@@ -106,6 +106,7 @@ namespace dxvk {
   
   D3D11DepthStencilView::~D3D11DepthStencilView() {
     ResourceReleasePrivate(m_resource);
+    m_resource = nullptr;
   }
   
   
@@ -126,7 +127,7 @@ namespace dxvk {
     if (riid == __uuidof(ID3D10DeviceChild)
      || riid == __uuidof(ID3D10View)
      || riid == __uuidof(ID3D10DepthStencilView)) {
-      *ppvObject = ref(this);
+      *ppvObject = ref(&m_d3d10);
       return S_OK;
     }
     
