@@ -116,6 +116,7 @@ namespace dxvk {
   
   D3D11RenderTargetView::~D3D11RenderTargetView() {
     ResourceReleasePrivate(m_resource);
+    m_resource = nullptr;
   }
   
   
@@ -137,7 +138,7 @@ namespace dxvk {
     if (riid == __uuidof(ID3D10DeviceChild)
      || riid == __uuidof(ID3D10View)
      || riid == __uuidof(ID3D10RenderTargetView)) {
-      *ppvObject = ref(this);
+      *ppvObject = ref(&m_d3d10);
       return S_OK;
     }
     
