@@ -204,37 +204,10 @@ namespace dxvk {
 
   bool IsDepthFormat(D3D9Format Format);
 
-  inline bool operator == (const D3DVIEWPORT9& a, const D3DVIEWPORT9& b) {
-    return a.X      == b.X      &&
-           a.Y      == b.Y      &&
-           a.Width  == b.Width  &&
-           a.Height == b.Height &&
-           a.MinZ   == b.MinZ   &&
-           a.MaxZ   == b.MaxZ;
-  }
+  bool IsDepthStencilFormat(D3D9Format Format);
 
-  inline bool operator != (const D3DVIEWPORT9& a, const D3DVIEWPORT9& b) {
-    return !(a == b);
-  }
+  bool IsLockableDepthStencilFormat(D3D9Format Format);
 
-  inline bool operator == (const RECT& a, const RECT& b) {
-    return a.left   == b.left  &&
-           a.right  == b.right &&
-           a.top    == b.top   &&
-           a.bottom == b.bottom;
-  }
-
-  inline bool operator != (const RECT& a, const RECT& b) {
-    return !(a == b);
-  }
-
-  inline bool operator == (const POINT& a, const POINT& b) {
-    return a.x == b.x && a.y == b.y;
-  }
-
-  inline bool operator != (const POINT& a, const POINT& b) {
-    return !(a == b);
-  }
 
   inline bool IsPoolManaged(D3DPOOL Pool) {
     return Pool == D3DPOOL_MANAGED || Pool == D3DPOOL_MANAGED_EX;
@@ -295,4 +268,47 @@ namespace dxvk {
     return D3D9TextureStageStateTypes(Type - 1);
   }
 
-}
\ No newline at end of file
+}
+
+
+inline bool operator == (const D3DVIEWPORT9& a, const D3DVIEWPORT9& b) {
+  return a.X      == b.X      &&
+         a.Y      == b.Y      &&
+         a.Width  == b.Width  &&
+         a.Height == b.Height &&
+         a.MinZ   == b.MinZ   &&
+         a.MaxZ   == b.MaxZ;
+}
+
+inline bool operator != (const D3DVIEWPORT9& a, const D3DVIEWPORT9& b) {
+  return !(a == b);
+}
+
+inline bool operator == (const RECT& a, const RECT& b) {
+  return a.left   == b.left  &&
+         a.right  == b.right &&
+         a.top    == b.top   &&
+         a.bottom == b.bottom;
+}
+
+inline bool operator != (const RECT& a, const RECT& b) {
+  return !(a == b);
+}
+
+inline bool operator == (const POINT& a, const POINT& b) {
+  return a.x == b.x && a.y == b.y;
+}
+
+inline bool operator != (const POINT& a, const POINT& b) {
+  return !(a == b);
+}
+
+inline bool operator == (const D3DDISPLAYMODEEX& a, const D3DDISPLAYMODEEX& b) {
+  return a.Size             == b.Size             &&
+         a.Width            == b.Width            &&
+         a.Height           == b.Height           &&
+         a.RefreshRate      == b.RefreshRate      &&
+         a.Format           == b.Format           &&
+         a.ScanLineOrdering == b.ScanLineOrdering;
+}
+
