@@ -4,6 +4,8 @@
 
 #include "../util/util_misc.h"
 
+#include <d3d12.h>
+
 namespace dxvk {
   
   DxgiSwapChain::DxgiSwapChain(
@@ -11,14 +13,17 @@ namespace dxvk {
           IDXGIVkSwapChain*           pPresenter,
           HWND                        hWnd,
     const DXGI_SWAP_CHAIN_DESC1*      pDesc,
-    const DXGI_SWAP_CHAIN_FULLSCREEN_DESC*  pFullscreenDesc)
+    const DXGI_SWAP_CHAIN_FULLSCREEN_DESC*  pFullscreenDesc,
+          IUnknown*                   pDevice)
   : m_factory   (pFactory),
     m_window    (hWnd),
     m_desc      (*pDesc),
     m_descFs    (*pFullscreenDesc),
     m_presentId (0u),
     m_presenter (pPresenter),
-    m_monitor   (wsi::getWindowMonitor(m_window)) {
+    m_monitor   (wsi::getWindowMonitor(m_window)),
+    m_is_d3d12(SUCCEEDED(pDevice->QueryInterface(__uuidof(ID3D12CommandQueue), reinterpret_cast<void**>(&Com<ID3D12CommandQueue>())))) {
+
     if (FAILED(m_presenter->GetAdapter(__uuidof(IDXGIAdapter), reinterpret_cast<void**>(&m_adapter))))
       throw DxvkError("DXGI: Failed to get adapter for present device");
 
@@ -243,7 +248,9 @@ namespace dxvk {
           BOOL*         pFullscreen,
           IDXGIOutput** ppTarget) {
     HRESULT hr = S_OK;
-    
+
+    if (!m_is_d3d12 && !m_descFs.Windowed && wsi::isOccluded(m_window))
+      SetFullscreenState(FALSE, nullptr);
     if (pFullscreen != nullptr)
       *pFullscreen = !m_descFs.Windowed;
     
@@ -325,6 +332,10 @@ namespace dxvk {
     if (SyncInterval > 4)
       return DXGI_ERROR_INVALID_CALL;
 
+    if ((m_desc.SwapEffect == DXGI_SWAP_EFFECT_DISCARD || m_desc.SwapEffect == DXGI_SWAP_EFFECT_SEQUENTIAL) && wsi::isMinimized(m_window))
+      return DXGI_STATUS_OCCLUDED;
+    bool occluded = !m_descFs.Windowed && wsi::isOccluded(m_window) && !wsi::isMinimized(m_window);
+
     auto options = m_factory->GetOptions();
 
     if (options->syncInterval >= 0)
@@ -342,7 +353,7 @@ namespace dxvk {
     }
 
     if (PresentFlags & DXGI_PRESENT_TEST)
-      return hr;
+      return hr == S_OK && occluded ? DXGI_STATUS_OCCLUDED : hr;
 
     if (hr == S_OK) {
 
@@ -365,6 +376,11 @@ namespace dxvk {
         monitorData->FrameStats.PresentRefreshCount = monitorData->FrameStats.SyncRefreshCount + computeRefreshCount(t0, t1, refreshPeriod);
         ReleaseMonitorData();
       }
+      if (occluded) {
+        if (!(PresentFlags & DXGI_PRESENT_TEST))
+          SetFullscreenState(FALSE, nullptr);
+        hr = DXGI_STATUS_OCCLUDED;
+      }
     }
 
     return hr;
@@ -658,6 +674,12 @@ namespace dxvk {
 
 
   HRESULT DxgiSwapChain::EnterFullscreenMode(IDXGIOutput1* pTarget) {
+    if (m_ModeChangeInProgress) {
+      Logger::warn("Nested EnterFullscreenMode");
+      return DXGI_STATUS_MODE_CHANGE_IN_PROGRESS;
+    }
+    scoped_bool in_progress(m_ModeChangeInProgress);
+
     Com<IDXGIOutput1> output = pTarget;
 
     if (!wsi::isWindow(m_window))
@@ -718,6 +740,12 @@ namespace dxvk {
   
   
   HRESULT DxgiSwapChain::LeaveFullscreenMode() {
+    if (m_ModeChangeInProgress) {
+      Logger::warn("Nested LeaveFullscreenMode");
+      return DXGI_STATUS_MODE_CHANGE_IN_PROGRESS;
+    }
+    scoped_bool in_progress(m_ModeChangeInProgress);
+
     if (FAILED(RestoreDisplayMode(m_monitor)))
       Logger::warn("DXGI: LeaveFullscreenMode: Failed to restore display mode");
     
@@ -731,7 +759,7 @@ namespace dxvk {
       SetGammaControl(0, nullptr);
       ReleaseMonitorData();
     }
-    
+
     // Restore internal state
     m_descFs.Windowed = TRUE;
     m_target  = nullptr;
