@@ -2,11 +2,16 @@
 
 namespace dxvk {
   
-  DxvkDeviceFilter::DxvkDeviceFilter(DxvkDeviceFilterFlags flags)
+  DxvkDeviceFilter::DxvkDeviceFilter(
+          DxvkDeviceFilterFlags flags,
+    const DxvkOptions&          options)
   : m_flags(flags) {
     m_matchDeviceName = env::getEnvVar("DXVK_FILTER_DEVICE_NAME");
-    
-    if (m_matchDeviceName.size() != 0)
+
+    if (m_matchDeviceName.empty())
+      m_matchDeviceName = options.deviceFilter;
+
+    if (!m_matchDeviceName.empty())
       m_flags.set(DxvkDeviceFilterFlag::MatchDeviceName);
   }
   
@@ -17,10 +22,10 @@ namespace dxvk {
   
   
   bool DxvkDeviceFilter::testAdapter(const VkPhysicalDeviceProperties& properties) const {
-    if (properties.apiVersion < VK_MAKE_VERSION(1, 3, 0)) {
+    if (properties.apiVersion < VK_MAKE_API_VERSION(0, 1, 3, 0)) {
       Logger::warn(str::format("Skipping Vulkan ",
-        VK_VERSION_MAJOR(properties.apiVersion), ".",
-        VK_VERSION_MINOR(properties.apiVersion), " adapter: ",
+        VK_API_VERSION_MAJOR(properties.apiVersion), ".",
+        VK_API_VERSION_MINOR(properties.apiVersion), " adapter: ",
         properties.deviceName));
       return false;
     }
