@@ -49,6 +49,7 @@ namespace dxvk {
    * be persistently mapped.
    */
   struct DxvkDeviceMemory {
+    VkBuffer              buffer     = VK_NULL_HANDLE;
     VkDeviceMemory        memHandle  = VK_NULL_HANDLE;
     void*                 memPointer = nullptr;
     VkDeviceSize          memSize    = 0;
@@ -66,7 +67,6 @@ namespace dxvk {
   struct DxvkMemoryHeap {
     VkMemoryHeap      properties;
     DxvkMemoryStats   stats;
-    VkDeviceSize      budget;
   };
 
 
@@ -84,6 +84,9 @@ namespace dxvk {
     VkMemoryType      memType;
     uint32_t          memTypeId;
 
+    VkDeviceSize      chunkSize;
+    VkBufferUsageFlags bufferUsage;
+
     std::vector<Rc<DxvkMemoryChunk>> chunks;
   };
   
@@ -103,6 +106,7 @@ namespace dxvk {
       DxvkMemoryAllocator*  alloc,
       DxvkMemoryChunk*      chunk,
       DxvkMemoryType*       type,
+      VkBuffer              buffer,
       VkDeviceMemory        memory,
       VkDeviceSize          offset,
       VkDeviceSize          length,
@@ -122,6 +126,16 @@ namespace dxvk {
       return m_memory;
     }
     
+    /**
+     * \brief Buffer object
+     *
+     * Global buffer covering the entire memory allocation.
+     * \returns Buffer object
+     */
+    VkBuffer buffer() const {
+      return m_buffer;
+    }
+
     /**
      * \brief Offset into device memory
      * 
@@ -158,15 +172,24 @@ namespace dxvk {
      * \returns \c true if this slice points to actual device
      *          memory, and \c false if it is undefined.
      */
-    operator bool () const {
+    explicit operator bool () const {
       return m_memory != VK_NULL_HANDLE;
     }
-    
+
+    /**
+     * \brief Queries global buffer usage flags
+     * \returns Global buffer usage flags, if any
+     */
+    VkBufferUsageFlags getBufferUsage() const {
+      return m_buffer ? m_type->bufferUsage : 0u;
+    }
+
   private:
     
     DxvkMemoryAllocator*  m_alloc  = nullptr;
     DxvkMemoryChunk*      m_chunk  = nullptr;
     DxvkMemoryType*       m_type   = nullptr;
+    VkBuffer              m_buffer = VK_NULL_HANDLE;
     VkDeviceMemory        m_memory = VK_NULL_HANDLE;
     VkDeviceSize          m_offset = 0;
     VkDeviceSize          m_length = 0;
@@ -212,6 +235,14 @@ namespace dxvk {
     
     ~DxvkMemoryChunk();
 
+    /**
+     * \brief Queries chunk size
+     * \returns Chunk size
+     */
+    VkDeviceSize size() const {
+      return m_memory.memSize;
+    }
+
     /**
      * \brief Allocates memory from the chunk
      * 
@@ -305,6 +336,9 @@ namespace dxvk {
     friend class DxvkMemoryChunk;
 
     constexpr static VkDeviceSize SmallAllocationThreshold = 256 << 10;
+
+    constexpr static VkDeviceSize MinChunkSize =   4ull << 20;
+    constexpr static VkDeviceSize MaxChunkSize = 256ull << 20;
   public:
     
     DxvkMemoryAllocator(DxvkDevice* device);
@@ -343,16 +377,38 @@ namespace dxvk {
       return m_memHeaps[heap].stats;
     }
     
+    /**
+     * \brief Queries buffer memory requirements
+     *
+     * Can be used to get memory requirements without having
+     * to create a buffer object first.
+     * \param [in] createInfo Buffer create info
+     * \param [in,out] memoryRequirements Memory requirements
+     */
+    bool getBufferMemoryRequirements(
+      const VkBufferCreateInfo&     createInfo,
+            VkMemoryRequirements2&  memoryRequirements) const;
+
+    /**
+     * \brief Queries image memory requirements
+     *
+     * Can be used to get memory requirements without having
+     * to create an image object first.
+     * \param [in] createInfo Image create info
+     * \param [in,out] memoryRequirements Memory requirements
+     */
+    bool getImageMemoryRequirements(
+      const VkImageCreateInfo&      createInfo,
+            VkMemoryRequirements2&  memoryRequirements) const;
+
   private:
 
     DxvkDevice*                                     m_device;
     VkPhysicalDeviceMemoryProperties                m_memProps;
     
     dxvk::mutex                                     m_mutex;
-    std::array<DxvkMemoryHeap, VK_MAX_MEMORY_HEAPS> m_memHeaps;
-    std::array<DxvkMemoryType, VK_MAX_MEMORY_TYPES> m_memTypes;
-
-    VkDeviceSize                                    m_maxChunkSize;
+    std::array<DxvkMemoryHeap, VK_MAX_MEMORY_HEAPS> m_memHeaps = { };
+    std::array<DxvkMemoryType, VK_MAX_MEMORY_TYPES> m_memTypes = { };
 
     uint32_t m_sparseMemoryTypes = 0u;
 
@@ -389,8 +445,14 @@ namespace dxvk {
     
     VkDeviceSize pickChunkSize(
             uint32_t              memTypeId,
+            VkDeviceSize          requiredSize,
             DxvkMemoryFlags       hints) const;
 
+    void adjustChunkSize(
+            uint32_t              memTypeId,
+            VkDeviceSize          allocatedSize,
+            DxvkMemoryFlags       hints);
+
     bool shouldFreeChunk(
       const DxvkMemoryType*       type,
       const Rc<DxvkMemoryChunk>&  chunk) const;
@@ -405,8 +467,7 @@ namespace dxvk {
     uint32_t determineSparseMemoryTypes(
             DxvkDevice*           device) const;
 
-    VkDeviceSize determineMaxChunkSize(
-            DxvkDevice*           device) const;
+    void determineBufferUsageFlagsPerMemoryType();
 
     void logMemoryError(
       const VkMemoryRequirements& req) const;
