@@ -1,5 +1,7 @@
 #pragma once
 
+#include <queue>
+#include <unordered_map>
 #include <unordered_set>
 
 #include "spirv_code_buffer.h"
@@ -59,7 +61,7 @@ namespace dxvk {
 
     ~SpirvModule();
     
-    SpirvCodeBuffer compile() const;
+    SpirvCodeBuffer compile();
 
     size_t getInsertionPtr() {
       return m_code.getInsertionPtr();
@@ -217,6 +219,9 @@ namespace dxvk {
     uint32_t constUndef(
             uint32_t                typeId);
     
+    uint32_t constNull(
+            uint32_t                typeId);
+    
     uint32_t lateConst32(
             uint32_t                typeId);
 
@@ -951,6 +956,14 @@ namespace dxvk {
             uint32_t                resultType,
             uint32_t                operand);
 
+    uint32_t opRawAccessChain(
+            uint32_t                resultType,
+            uint32_t                base,
+            uint32_t                stride,
+            uint32_t                index,
+            uint32_t                offset,
+            uint32_t                operand);
+
     uint32_t opReflect(
             uint32_t                resultType,
             uint32_t                incident,
@@ -1315,6 +1328,10 @@ namespace dxvk {
     bool isInterfaceVar(
             spv::StorageClass       sclass) const;
 
+    void classifyBlocks(
+            std::unordered_set<uint32_t>& reachableBlocks,
+            std::unordered_set<uint32_t>& mergeBlocks);
+
   };
   
 }
\ No newline at end of file
