@@ -17,6 +17,35 @@ namespace dxvk {
 
 
   const static ProfileList g_profiles = {{
+    /**********************************************/
+    /* D3D12 GAMES (vkd3d-proton with dxvk dxgi)  */
+    /**********************************************/
+
+    /* Diablo 4 - Will complain about missing  *
+     * GPU unless dxgi Id match actual GPU Id  */
+    { R"(\\Diablo IV\.exe$)", {{
+      { "dxgi.hideNvidiaGpu",               "False"  },
+    }} },
+    /* Ratchet & Clank: Rift Apart - does not allow
+     * enabling ray tracing if it sees an AMD GPU. */
+    { R"(\\RiftApart\.exe$)", {{
+      { "dxgi.hideNvidiaGpu",               "False" },
+    }} },
+    /* Persona 3 Reload - disables vsync by default and
+     * runs into severe frame latency issues on Deck. */
+    { R"(\\P3R\.exe$)", {{
+      { "dxgi.syncInterval",                "1" },
+    }} },
+
+    /**********************************************/
+    /* D3D11 GAMES                                */
+    /**********************************************/
+
+    /* Batman Arkham Knight - doesn't like intel vendor id 
+      (refuses to boot if vendor isn't 0x10de or 0x1002)  */
+    { R"(\\BatmanAK\.exe$)", {{
+      { "dxgi.hideIntelGpu",                "True" },
+    }} },
     /* Assassin's Creed Syndicate: amdags issues  */
     { R"(\\ACS\.exe$)", {{
       { "dxgi.customVendorId",              "10de" },
@@ -358,30 +387,6 @@ namespace dxvk {
     { R"(\\SnowRunner\.exe$)", {{
       { "d3d11.dcSingleUseMode",            "False" },
     }} },
-    /* Rockstar Games Launcher                    */
-    { R"(\\Rockstar Games\\Launcher\\Launcher\.exe$)", {{
-      { "dxvk.maxChunkSize",                "1"   },
-    }} },
-    /* Rockstar Social Club                       */
-    { R"(\\Rockstar Games\\Social Club\\SocialClubHelper\.exe$)", {{
-      { "dxvk.maxChunkSize",                "1"   },
-    }} },
-    /* EA Desktop App                             */
-    { R"(\\EADesktop\.exe$)", {{
-      { "dxvk.maxChunkSize",                "1"   },
-    }} },
-    /* Origin app (legacy EA Desktop)             */
-    { R"(\\Origin\\(Origin|OriginWebHelperService)\.exe$)", {{
-      { "dxvk.maxChunkSize",                "1"   },
-    }} },
-    /* Ubisoft Connect (UPlay)                    */
-    { R"(\\Ubisoft\\Ubisoft Game Launcher\\(UbisoftConnect|upc)\.exe$)", {{
-      { "dxvk.maxChunkSize",                "1"   },
-    }} },
-    /* GOG Galaxy                                 */
-    { R"(\\GalaxyClient\.exe$)", {{
-      { "dxvk.maxChunkSize",                "1"   },
-    }} },
     /* Fallout 76
      * Game tries to be too "smart" and changes sync
      * interval based on performance (in fullscreen)
@@ -397,10 +402,6 @@ namespace dxvk {
     { R"(\\Fallout76\.exe$)", {{
       { "dxgi.syncInterval",                "1" },
     }} },
-    /* Blizzard Entertainment Battle.net          */
-    { R"(\\Battle\.net\.exe$)", {{
-      { "dxvk.maxChunkSize",                "1" },
-    }} },
     /* Bladestorm Nightmare                       *
      * Game speed increases when above 60 fps in  *
      * the tavern area                            */
@@ -432,31 +433,21 @@ namespace dxvk {
       { "d3d11.exposeDriverCommandLists",   "False" },
       { "dxgi.hideNvidiaGpu",               "False" },
     }} },
-    /* Red Faction Guerrilla Re-Mars-tered        *
-     * Broken skybox                              */
-    { R"(\\rfg\.exe$)", {{
-      { "d3d11.longMad",                  "True"    },
-    }} },
-    /* Guild Wars 2 - Fixes invisibility effect   *
-     * flicker when invariantPosition is enabled  */
-    { R"(\\Gw2-64\.exe$)", {{
-      { "d3d11.longMad",                  "True"    },
-    }} },
-    /* Ghostbusters: The Video Game Remastered    *
-     * Flickering on character faces              */
-    { R"(\\ghost\.exe$)", {{
-      { "d3d11.longMad",                  "True"    },
-    }} },
-    /* Watch_Dogs series - Some objects flicker  */
-    { R"(\\watch(_)?dogs(2|Legion)?\.exe$)", {{
-      { "d3d11.longMad",                  "True"    },
-    }} },
     /* Crysis 1/Warhead - Game bug in d3d10 makes *
      * it select lowest supported refresh rate    */
     { R"(\\Crysis(64)?\.exe$)", {{
       { "d3d9.maxFrameRate",              "-1"      },
       { "dxgi.maxFrameRate",              "-1"      },
     }} },
+    /* EDF6 - possible race condition?            */
+    { R"(\\EDF6\.exe$)", {{
+      { "d3d11.enableContextLock",          "True" },
+    }} },
+    /* Kena: Bridge of Spirits: intel water       * 
+      flickering issues                           */
+    { R"(\\Kena-Win64-Shipping\.exe$)", {{
+      { "dxgi.hideIntelGpu",                 "True" },
+    }} },
 
     /**********************************************/
     /* D3D9 GAMES                                 */
@@ -566,14 +557,15 @@ namespace dxvk {
     }} },
     /* GTA IV (NVAPI)                             */
     /* Also thinks we're always on Intel          *
-     * and will report/use bad amounts of VRAM.
+     * and will report/use bad amounts of VRAM
+     * if we report more than 128 MB of VRAM.
      * Disabling support for DF texture formats
      * makes the game use a better looking render
      * path for mirrors.
      * Also runs into issues after alt-tabbing.   */
     { R"(\\(GTAIV|EFLC)\.exe$)", {{
       { "d3d9.customVendorId",              "1002" },
-      { "dxgi.emulateUMA",                  "True" },
+      { "dxgi.maxDeviceMemory",             "128" },
       { "d3d9.supportDFFormats",            "False" },
       { "d3d9.deviceLossOnFocusLoss",       "True" },
     }} },
@@ -583,7 +575,6 @@ namespace dxvk {
      * like a game bug that d3d9 drivers work     *
      * around.                                    */
     { R"(\\(BF2|BF2142)\.exe$)", {{
-      { "d3d9.longMad",                     "True" },
       { "d3d9.deviceLossOnFocusLoss",       "True" },
       { "d3d9.countLosableResources",       "False"},
     }} },
@@ -599,7 +590,6 @@ namespace dxvk {
     { R"(\\(trl|tra|tru)\.exe$)", {{
       { "d3d9.cachedDynamicBuffers",        "True" },
       { "d3d9.maxFrameRate",                "60" },
-      { "d3d9.longMad",                     "True" },
     }} },
     /* Everquest                                 */
     { R"(\\eqgame\.exe$)", {{
@@ -631,6 +621,10 @@ namespace dxvk {
     { R"(\\k2\.exe$)", {{
       { "d3d9.memoryTrackTest",             "True" },
     }} },
+    /* Time Leap Paradise SUPER LIVE             */
+    { R"(\\tlpsl\.exe$)", {{
+      { "d3d9.deferSurfaceCreation",        "True" },
+    }} },
     /* Ninja Gaiden Sigma 1/2                    */
     { R"(\\NINJA GAIDEN SIGMA(2)?\.exe$)", {{
       { "d3d9.deferSurfaceCreation",        "True" },
@@ -949,47 +943,44 @@ namespace dxvk {
     }} },
     /* Battle for Middle-earth 2 and expansion   *
      * Slowdowns in certain scenarios            */
-    { R"(\\(The Battle for Middle-earth (\(tm\))? II( Demo)?|The Lord of the Rings, The Rise of the Witch-king)\\game\.dat$)", {{
+    { R"(\\(The Battle for Middle-earth( \(tm\))? II( Demo)?)"
+      R"(|The Lord of the Rings, The Rise of the Witch-king)\\game\.dat$)", {{
       { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
-    /* WRC4 - Audio breaks above 60fps */
+    /* WRC4 - Audio breaks above 60fps           */
     { R"(\\WRC4\.exe$)", {{
       { "d3d9.maxFrameRate",                "60" },
     }} },
-
-
-    /**********************************************/
-    /* D3D12 GAMES (vkd3d-proton with dxvk dxgi)  */
-    /**********************************************/
-
-    /* Diablo 4 - Will complain about missing  *
-     * GPU unless dxgi Id match actual GPU Id  */
-    { R"(\\Diablo IV\.exe$)", {{
-      { "dxgi.hideNvidiaGpu",               "False"  },
+    /* Splinter Cell Conviction - Alt-tab black  *
+     * screen and unsupported GPU complaint      */
+    { R"(\\conviction_game\.exe$)", {{
+      { "d3d9.deviceLossOnFocusLoss",       "True" },
+      { "dxgi.customVendorId",              "10de" },
+      { "dxgi.customDeviceId",              "05e0" },
+      { "dxgi.customDeviceDesc",            "GeForce GTX 295" },
     }} },
-    /* WILD HEARTS™️                            *
-     * D3D12 title using D3D11 device for      *
-     * media texture creation, whereby a large *
-     * chunk size only slows down media        *
-     * initialization                          */
-    { R"(\\WILD HEARTS(_Trial)?\.exe$)", {{
-      { "dxvk.maxChunkSize",                 "4" },
+    /* Resident Evil: Operation Raccoon City     */
+    { R"(\\RaccoonCity\.exe$)", {{
+      { "d3d9.textureMemory",               "0" },
     }} },
-    /* Ratchet & Clank: Rift Apart - does not allow
-     * enabling ray tracing if it sees an AMD GPU. */
-    { R"(\\RiftApart\.exe$)", {{
-      { "dxgi.hideNvidiaGpu",               "False" },
+    /* APB: Reloaded                               *
+     * Fixes frametime jumps when shooting         */
+    { R"(\\APB\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
-    /* Metro Exodus Enhanced Edition picks GPU adapters
-     * by available VRAM, which causes issues on some
-     * systems with integrated graphics. */
-    { R"(\\Metro Exodus Enhanced Edition\\MetroExodus\.exe$)", {{
-      { "dxvk.hideIntegratedGraphics",      "True" },
+    /* Battle Mages - helps CPU bound perf         */
+    { R"(\\Battle Mages\\mages\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
-    /* Persona 3 Reload - disables vsync by default and
-     * runs into severe frame latency issues on Deck. */
-    { R"(\\P3R\.exe$)", {{
-      { "dxgi.syncInterval",                "1" },
+    /* Prince of Persia (2008) - Can get stuck     *
+     * during loading at very high fps             */
+    { R"(\\PrinceOfPersia_Launcher\.exe$)", {{
+      { "d3d9.maxFrameRate",                 "240" },
+    }} },
+    /* F.E.A.R 1 & expansions                      *
+     * Graphics glitches at very high fps          */
+    { R"(\\FEAR(MP|XP|XP2)?\.exe$)", {{
+      { "d3d9.maxFrameRate",                 "360" },
     }} },
 
     /**********************************************/
@@ -1005,11 +996,6 @@ namespace dxvk {
     { R"(\\indy\.exe$)", {{
       { "d3d9.enableDialogMode",            "True" },
     }} },
-    /* Tom Clancy's Splinter Cell                *
-     * Supports shadow buffers                   */
-    { R"(\\splintercell\.exe$)", {{
-      { "d3d8.useShadowBuffers",            "True" },
-    }} },
     /* Anito: Defend a Land Enraged              */
     { R"(\\Anito\.exe$)", {{
       { "d3d9.memoryTrackTest",             "True" },
@@ -1122,6 +1108,44 @@ namespace dxvk {
     { R"(\\(nations|patriots)\.exe$)", {{
       { "d3d9.deviceLossOnFocusLoss",       "True" },
     }} },
+    /* Inquisitor (2009)                          *
+     * Leaks a resource when alt-tabbing          */
+    { R"(\\Inquisitor\.exe$)", {{
+      { "d3d9.countLosableResources",      "False" },
+    }} },
+    /* Art of Murder FBI Confidential - CPU perf  */
+    { R"(\\Art of Murder - FBI Confidential\\game\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
+    }} },
+    /* Max Payne 1 - Stalls waiting for an index buffer */
+    { R"(\\MaxPayne\.exe$)", {{
+      { "d3d9.allowDirectBufferMapping",   "False" },
+    }} },
+    /* Z: Steel Soldiers                          */
+    { R"(\\z2\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
+    }} },
+    /* FIFA Football 2003                         */
+    { R"(\\fifa2003(demo)?\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
+    }} },
+    /* Splinter Cell: Pandora Tomorrow            *
+     * Broken inputs and physics above 60 FPS     */
+    { R"(\\SplinterCell2\.exe$)", {{
+      { "d3d9.maxFrameRate",                  "60" },
+    }} },
+    /* Chrome: Gold Edition                       *
+     * Broken character model motion at high FPS  */
+    { R"(\\Chrome(Single|Net)\.exe$)", {{
+      { "d3d9.maxFrameRate",                  "60" },
+    }} },
+    /* Rayman 3: Hoodlum Havoc                    *
+     * Missing geometry and textures without      *
+     * legacy DISCARD behavior                    */
+    { R"(\\Rayman3\.exe$)", {{
+      { "d3d9.allowDirectBufferMapping",   "False" },
+      { "d3d8.forceLegacyDiscard",          "True" },
+    }} },
   }};
 
 
